/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "serverclient.h"
#include "util/memory.h"
#include <sys/socket.h>
#include "console/console.h"

errorret_t serverClientAccept(
  serverclient_t *client,
  const int32_t socket
) {
  memoryZero(client, sizeof(serverclient_t));
  client->socket = socket;
  client->state = SERVER_CLIENT_STATE_ACCEPTING;

  // Set timeout to 20 seconds
  client->timeout.tv_usec = SERVER_CLIENT_TIMEOUT * 1000;

  // Create a thread for the client
  if(pthread_create(&client->thread, NULL, serverClientThread, client) != 0) {
    client->state = SERVER_CLIENT_STATE_DISCONNECTED;
    return error("Failed to create client thread");
  }
}

errorret_t serverClientSend(
  serverclient_t * client,
  const uint8_t *data,
  const size_t len
) {
  if(client->state != SERVER_CLIENT_STATE_ACCEPTING) {
    return error("Client is not accepting");
  }
  ssize_t sent = send(client->socket, data, len, 0);
  if(sent < 0) return error("Failed to send data");
  return ERROR_OK;
}

errorret_t serverClientSendString(
  serverclient_t * client,
  const char_t *data
) {
  return serverClientSend(client, (const uint8_t *)data, strlen(data));
}

ssize_t serverClientReceive(
  serverclient_t * client,
  uint8_t *buffer,
  const size_t len
) {
  if(client->state != SERVER_CLIENT_STATE_ACCEPTING) return -1;
  ssize_t received = recv(client->socket, buffer, len, 0);
  if(received < 0) return received;
  return received;
}

void serverClientClose(serverclient_t *client) {
  client->state = SERVER_CLIENT_STATE_DISCONNECTED;
  close(client->socket);
  pthread_join(client->thread, NULL);
  client->thread = 0;
  client->socket = -1;
  consolePrint("Client disconnected.");
}

void *serverClientThread(void *arg) {
  serverclient_t *client = (serverclient_t *)arg;
  char_t buffer[1024];
  ssize_t read;

  printf("Client thread started for socket %d.\n", client->socket);

  // Set socket timeout
  setsockopt(
    client->socket,
    SOL_SOCKET,
    SO_RCVTIMEO,
    &client->timeout,
    sizeof(client->timeout)
  );

  // Send welcome message
  serverClientSendString(client, "DUSK|"DUSK_VERSION);

  // Receive version from client
  read = serverClientReceive(client, buffer, sizeof(buffer));

  // First 5 bytes should be "DUSK|"
  if(read < 5 || strncmp(buffer, "DUSK|", 5) != 0) {
    serverClientSendString(client, "ERR|Invalid version (0)");
    serverClientClose(client);
    return NULL;
  }

  // Next (up to X bytes where X is the length of the version string)
  // should be the version string
  buffer[read] = '\0'; // Null-terminate the string


  printf("Received version: %s\n", buffer);
  
  serverClientClose(client);
  return NULL;
}