/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "error/error.h"

#define SERVER_CLIENT_TIMEOUT 20000 // 20 seconds

typedef enum {
  SERVER_CLIENT_STATE_DISCONNECTED,
  SERVER_CLIENT_STATE_ACCEPTING,
} serverclientstate_t;

typedef struct {
  int32_t socket;
  serverclientstate_t state;
  pthread_t thread; // Add thread field
  struct timeval timeout; // Add timeout field
} serverclient_t;

/**
 * Accepts an incoming connection from a server client.
 */
errorret_t serverClientAccept(
  serverclient_t *client,
  const int32_t socket
);

/**
 * Sends data to a server client.
 * 
 * @param client Pointer to the server client structure.
 * @param data Pointer to the data to send.
 * @param len Length of the data to send.
 * @return Error code indicating success or failure.
 */
errorret_t serverClientSend(
  serverclient_t * client,
  const uint8_t *data,
  const size_t len
);

/**
 * Sends a string to a server client.
 * 
 * @param client Pointer to the server client structure.
 * @param data Pointer to the string to send.
 * @return Error code indicating success or failure.
 */
errorret_t serverClientSendString(
  serverclient_t * client,
  const char_t *data
);

/**
 * Receives data from a server client.
 * 
 * @param client Pointer to the server client structure.
 * @param buffer Pointer to the buffer to store received data.
 * @param len Max length of the buffer.
 * @return Number of bytes received. 0 or less indicates an error.
 */
ssize_t serverClientReceive(
  serverclient_t * client,
  uint8_t *buffer,
  const size_t len
);

/**
 * Closes the connection to a server client.
 */
void serverClientClose(
  serverclient_t *client
);

/**
 * Thread function for handling a server client.
 * 
 * @param arg Pointer to the server client structure.
 */
void *serverClientThread(void *arg);